//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://emule.sf.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#include "StdAfx.h"
#include "preferences.h"
#include <time.h>
#include "opcodes.h"

CPreferences::CPreferences(){
	srand((uint32)time(0)); // we need random numbers sometimes

	prefs = new Preferences_Struct;
	memset(prefs,0,sizeof(Preferences_Struct));
	//get application start directory
	char buffer[490];
	::GetModuleFileName(0, buffer, 490);
	LPTSTR pszFileName = _tcsrchr(buffer, '\\') + 1;
	*pszFileName = '\0';
	appdir = strdup(buffer);
	// load preferences.dat or set standart values
	char* fullpath = new char[strlen(appdir)+16];
	sprintf(fullpath,"%spreferences.dat",appdir);
	FILE* preffile = fopen(fullpath,"rb");
	delete[] fullpath;
	if (!preffile){
		SetStandartValues();
	}
	else{
		fread(prefs,sizeof(Preferences_Struct),1,preffile);
		if (ferror(preffile) || (prefs->version != PREFFILE_VERSION))
			SetStandartValues();
		fclose(preffile);
	}

	// shared directories
	fullpath = new char[strlen(appdir)+14];
	sprintf(fullpath,"%sshareddir.dat",appdir);
	CStdioFile* sdirfile = new CStdioFile();
	if (sdirfile->Open(fullpath,CFile::modeRead)){
		CString toadd;
		while (sdirfile->ReadString(toadd))
			shareddir_list.AddHead(toadd);
		sdirfile->Close();
	}
	delete sdirfile;
	delete[] fullpath;

	// make userhash random, this will cause problems with friend functions, but they aren't in yet anyway
	// however a random userhash provides more anon.
	for (int i = 0;i != 8; i++){ 
		uint32	random = rand();
		memcpy(&userhash[i*2],&random,2);
	}
	::CreateDirectory(GetIncomingDir(),0);
	::CreateDirectory(GetTempDir(),0);
}
void CPreferences::SetStandartValues(){
	prefs->version = PREFFILE_VERSION;
	prefs->maxupload = UNLIMITED;
	prefs->maxdownload = UNLIMITED;
	uint16 randnumber = (uint16)rand();
	sprintf(prefs->nick,"http://emule.sf.net");
	sprintf(prefs->incomingdir,"%sIncoming",appdir);
	sprintf(prefs->tempdir,"%sTemp",appdir);
	prefs->reconnect = true;
	prefs->scorsystem = true;
	prefs->maxconnections = 500;
	prefs->deadserver = 1;
	prefs->port = 4662;
	Save();
}

uint16 CPreferences::GetMaxDownload(){
	//dont be a Lam3r :)
	return ((GetMaxUpload() < 10)? GetMaxUpload()*3 : prefs->maxdownload);
}

bool CPreferences::Save(){
	bool error = false;
	char* fullpath = new char[strlen(appdir)+16];
	sprintf(fullpath,"%spreferences.dat",appdir);
	FILE* preffile = fopen(fullpath,"wb");
	delete[] fullpath;
	if (preffile){
		error = fwrite(prefs,sizeof(Preferences_Struct),1,preffile);
		fclose(preffile);
	}
	else
		error = true;
	fullpath = new char[strlen(appdir)+14];
	sprintf(fullpath,"%sshareddir.dat",appdir);
	CStdioFile* sdirfile = new CStdioFile();
	if (sdirfile->Open(fullpath,CFile::modeCreate|CFile::modeWrite)){

		for (POSITION pos = shareddir_list.GetHeadPosition();pos != 0;shareddir_list.GetNext(pos)){
			sdirfile->WriteString(shareddir_list.GetAt(pos).GetBuffer());
			sdirfile->Write("\n",1);
		}
		sdirfile->Close();
	}
	else
		error = true;
	delete[] fullpath;
	::CreateDirectory(GetIncomingDir(),0);
	::CreateDirectory(GetTempDir(),0);	
	return error;
}

CPreferences::~CPreferences(){
	delete[] appdir;
	delete prefs;
	//TODO: check if shareddir_list is properly deleted 
}
