//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://emule.sf.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#pragma once
#include "types.h"
#include "listensocket.h"
#include "partfile.h"

#define US_NONE				0
#define	US_PENDING			1
#define	US_ONUPLOADQUEUE	2
#define	US_UPLOADING		4
#define US_BANNED			6
#define US_ERROR			7
#define	US_WAITCALLBACK		8
#define	US_CONNECTING		9
#define	US_LOWTOLOWIP		10

#define	DS_NONE				0
#define	DS_DOWNLOADING		1
#define	DS_ONQUEUE			2
#define	DS_CONNECTING		3
#define	DS_WAITCALLBACK		4
#define	DS_NONEEDEDPARTS	5
#define DS_BANNED			6
#define DS_ERROR			7
#define	DS_CONNECTED		9
#define	DS_LOWTOLOWIP		10
#define	DS_REQHASHSET		11


class CClientReqSocket;

class CUpDownClient
{
	//friend class
public:
	//base
	CUpDownClient(CClientReqSocket* sender = 0);
	CUpDownClient(uint16 in_port, uint32 in_userid, uint32 in_serverup, uint16 in_serverport,CPartFile* in_reqfile);
	~CUpDownClient();
	void			Disconnected();
	void			ConnectionFailed();
	void			TryToConnect();
	void			ConnectionEstablished();
	uint32			GetUserID()					{return userid;}
	char*			GetUserName()				{return username;}
	uint32			GetIP()						{return userip;}
	bool			HasLowID()					{return (userid < 16777216);}
	char*			GetFullIP()					{return fulluserip;}
	uint32			GetUserPort()				{return userport;}
	uint32			GetTransferedUp()			{return transfered;}
	uint32			GetTransferedDown()			{return transfereddown;}
	uint32			GetServerIP()				{return serverip;}
	void			ProcessHelloAnswer(char* packet, uint32 size);
	void			ProcessHelloPacket(char* packet, uint32 size);
	void			SendHelloAnswer();
	void			SendHelloPacket();
	bool			Compare(CUpDownClient* tocomp);
	CClientReqSocket*	socket;

	//upload
	uint8			GetUploadState()			{return uploadstate;}
	void			SetUploadState(uint8 news)	{uploadstate = news;}
	uint32			GetWaitStartTime()			{return starttime;}
	uint32			GetWaitTime()				{return upstarttime-starttime;}
	bool			IsDownloading()				{return (uploadstate == US_UPLOADING);}
	bool			HasBlocks()					{return !blocksend_queue.IsEmpty();}
	uint32			GetDatarate()				{return datarate;}	
	uint32			GetScore(bool isdownloading = false);
	void			AddReqBlock(Requested_Block_Struct* reqblock);
	bool			CreateNextBlockPackage();
	void			SetUpStartTime(uint32 time = 0);
	void			SetWaitStartTime(uint32 time = 0);
	void			SendHashsetPacket(char* forfileid);
	uchar			reqfileid[16];
	uint32			SendBlockData(uint32 maxammount);

	//download
	uint8			GetDownloadState()			{return substate;}
	void			SetDownloadState(uint8 news);
	uint32			GetLastAskedTime()			{return lastaskedtime;}
	bool			IsPartAvailable(uint8 part)	{return	(part >= partcount)? 0:partstatus[part];} 	
	uint8*			GetPartStatus()				{return partstatus;}
	uint32			GetDownloadDatarate()		{return downdatarate;}
	void			DrawStatusBar(CDC* dc,RECT* rect,bool onlygreyrect);
	void			AskForDownload();
	void			SendFileRequest();
	void			ProcessFileInfo(char* packet,uint32 size);
	void			ProcessFileStatus(char* packet,uint32 size);
	void			ProcessHashSet(char* packet,uint32 size);
	bool			AddRequestForAnotherFile(CPartFile* file);
	void			SendBlockRequests();
	void			ProcessBlockPacket(char* packet, uint32 size);
	uint32			CalculateDownloadRate();
	uint16			GetAvailablePartCount();
	CPartFile*		reqfile;
private:
	// base
	void	Init();
	void	ProcessHelloTypePacket(CMemFile* data);
	void	SendHelloTypePacket(CMemFile* data);
	bool	isbotuser;
	bool	haslowid;
	bool	isfriend;
	bool	isemuleclient;
	int8	emuleversion;
	uint32	userip;
	uint32	serverip;
	uint32	userid;
	int16	userport;
	int16	serverport;
	uint32	clientversion;
	uint32	datarate;
	uint32	dataratems;
	uint32	count;
	char*	username;
	char	fulluserip[21];
	char	userhash[16];

	//upload
	uint32	transfered;
	uint8	uploadstate;
	uint32	starttime;
	uint32	upstarttime;
	int16	filepriority;
	uint32	maxsendallowed;
	CList<uint32,uint32>							 avarage_udr_list;
	CTypedPtrList<CPtrList, Packet*>				 blocksend_queue;
	CTypedPtrList<CPtrList, Requested_Block_Struct*> blockreqlist;
	CTypedPtrList<CPtrList, Requested_Block_Struct*> doneblocklist;
	//download
	uint8		substate;
	uint16		partcount;
	uint8*		partstatus;
	uint32		lastaskedtime;
	char*		clientfilename;
	uint32		transfereddown;
	uint32		downdatarate;
	uint32		downdataratems;
	uint8		failedconnection;
	bool		progressbarchanged;
	uint16		dcount;
	CList<uint32,uint32>							 avarage_ddr_list;
	CTypedPtrList<CPtrList, Pending_Block_Struct*>	 pendingblocks_list;
	CTypedPtrList<CPtrList, Requested_Block_Struct*> downloadblocks_list;
	CTypedPtrList<CPtrList, CPartFile*>				 otherrequests_list;
};

